;**************************************************************************************************
;
;   PIC_CAPACIMETER_II.ASM
;
;   CAPACIMETRO DIGITAL C/ PIC 16F628A E 555 MOSFET
;
;   CLAUDIO LRIOS
;
;   DATA: 24/03/2018
;
;   MEDE DE 1 PF A 9,9 F EM ESCALA AUTOMTICA, SENDO:
;
;   ESCALA A = 0 PF A 9,99 NF
;   ESCALA B = 10 NF A 9,99 UF
;   ESCALA C = 10 UF A 9,99 MF
;   ESCALA D=  10 MF A 9,99 F
;
;   USE XTAL DE 20 MHZ  NOS PINOS 15 E 16 DO PIC
;   PINO 4 COM RESISTOR DE 10K AO VCC
;   ANTES DE USAR PRECISA SER CALIBRADO COM CAPACITORES PADRES
;   E AJUSTAR OS 4 TRIMPOTS, UM PARA PARA CADA ESCALA 
;
;   USO DIDTICO
;  
;   BLOG PICSOURCE
;*************************************************************************************************
;ARQUIVOS HEADER 
    LIST P=16f628A , R=DEC
	INCLUDE "P16F628A.INC"

 
;PALAVRA DE CONFIGURAO
	__CONFIG _CP_OFF & _PWRTE_ON & _WDT_OFF & _HS_OSC  & _BODEN_ON & _LVP_OFF & _MCLRE_ON

;************************************************************************************************
;OPES DE COMPILAO DE ESCOLHA DO USURIO
;COMENTE OU DESCOMENTE E DEPOIS RECOMPILE USANDO O MPLAB-IDE

;#DEFINE ENGLISH    ;DESCOMENTE PARA MENSAGENS EM INGLS/ COMENTE PARA PORTUGUS

#DEFINE CAP_PINO6   ;DESCOMENTE SE USAR CAPACITOR DE 47uF NO PINO 6 
                    ;COM MENSAGEM DO BLOG SOMENTE AO LIGAR A ALIMENTAO
                    ;E NO QUANDO APERTA BOTO DE RESET/ZERAR

#DEFINE VIRGULA   ;DESCOMENTE PARA USAR VIRGULA AO INVS DE PONTO
;************************************************************************************************
;BANCOS
#DEFINE BANK0  BCF STATUS,RP0
#DEFINE  BANK1  BSF STATUS,RP0

;DEFINIO DE PINOS DO PIC LIGADOS AO TLC555
#DEFINE RESET_555  PORTA,3 ;PINO 2 OUT (RESETA 555)
#DEFINE OUT_555    PORTA,4 ;PINO 3 IN (PULSO A MEDIR)
#DEFINE DISP_555   PORTA,2 ;PINO 1 OUT (DISPARA MEDIO)  
#DEFINE ESC_NF     PORTA,1 ;PINO 18 OUT (ESCALA DE NANOFARADS)
#DEFINE ESC_UF     PORTA,0 ;PINO 17 OUT (ESCALA MICROFARADS)
#DEFINE ESC_MF     PORTB,3 ;PINO 9 OUT (ESCALA MILIFARADS)
#DEFINE V_BAT      PORTB,2 ;PINO 8 IN (TESTE DE TENSO DA BATERIA)

;DEFINIO DE PINOS DO PIC LIGADOS AO LCD 16X2
#DEFINE LCD_E      PORTB,0 ;PINO 6 
#DEFINE LCD_RS     PORTB,1 ;PINO 7  
;LCD_RW                    ;LIGAR AO GND DIRETO
#DEFINE LCD_D4     PORTB,4 ;PINO  10
#DEFINE LCD_D5     PORTB,5 ;PINO  11
#DEFINE LCD_D6     PORTB,6 ;PINO  12
#DEFINE LCD_D7     PORTB,7 ;PINO  13

;FLAGS
#DEFINE  FLAG_RS     R_FLAGS,0

;RAM
    CBLOCK 0X20
    TEMP1_L      ;TEMPORRIO L 
    TEMP1_H      ;TEMPORRIO H 
    TEMP1_L0     ;TEMPORRIO L 
    TEMP1_L1     ;TEMPORRIO H 
    CONT0        ;CONTADOR L  
    CONT1        ;CONTADOR H 
    DIV_0        ;DIVISO      
    DIV_1        ;DIVISO     
    RES_L        ;RESULTADO L/UNIDADE 
    RES_H        ;RESULTADO H  
    RES_L_SALVO  ;RESULTADO L SALVO
    RES_H_SALVO  ;RESULTADO H SALVO 
    RES1_L       ;RESULTADO L 
    RES1_H       ;RESULTADO H 
    RES1_L_SALVO ;RESULTADO DE RES1_L SALVO 
    RES1_H_SALVO ;RESULTADO DE RES1_H SALVO 
    RES2_L_SALVO ;RESULTADO L SALVO
    RES2_H_SALVO ;RESULTADO L SALVO
    DELAY_0      ;USADO EM DELAYS     
    DELAY_1      ;USADO EM DELAYS  
    DELAY_2      ;USADO EM DELAYS  
    DEZENAS      ;ARMAZENA DEZENAS
    CENTENAS     ;ARMAZENA CENTENAS     
    MILHARES     ;ARMAZENA MILHARES
    N_LEITURAS   ;NMERO DE LEITURAS (VALORES> mF)      
    ESCALA       ;0=PF OU 1={NF,UF,MF,F}    
    PREFIXO      ;PREFIXO U,N,P,M
    PONTO        ;PONTO (VIRGULA) 
    DADO         ;DADO PARA ENVIAR PARA LCD 
    R_FLAGS      ;REGISTRADOR DE FLAGS       
    ENDC

;======================================================================

       ORG 0X00 
       GOTO   START  ;VAI PARA O INCIO DO PROGRAMA
;====================================================================== 
       ORG 0X04
       RETFIE        ; NO FORAM USADAS INTERRUPES

;======================================================================
;           SUBROTINAS USADAS NO PROGRAMA
;======================================================================

;**********************************************************************
;     MENSAGENS NO LCD
;**********************************************************************
;============================================================
;ENVIA NOME DO APARELHO PARA PRIMEIRA LINHA
;============================================================
MENSAGEM_NAME: 
       BCF    FLAG_RS  ;DEFINE ENVIO DE COMANDO (RS=0)
       MOVLW  0x80
       MOVWF  DADO
       CALL   ENV_BYTE ;ENVIA COMANDO 'IR NA 2 LINHA'
       BSF    FLAG_RS ;DEFINE ENVIO DE DADOS (RS=1)
       MOVLW  'P'     ;ENVIA  'P'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'i'     ;ENVIA  'i'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'c'     ;ENVIA  'c'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '     ;ENVIA  ' '
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'C'     ;ENVIA  'C'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'a'     ;ENVIA  'a'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'p'    ;ENVIA   'P'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'a'     ;ENVIA  'a'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'c'     ;ENVIA  'c'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'i'     ;ENVIA  'i'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'm'    ;ENVIA   'm'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'e'     ;ENVIA  'e'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  't'     ;ENVIA  't'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'e'     ;ENVIA  'e'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'r'     ;ENVIA  'r'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '     ;ENVIA ESPAO ' '
       MOVWF  DADO
       CALL   ENV_BYTE
       RETURN
;============================================================
;ENVIA PARA 2 LINHA A MENSAGEM : ' Blog PicSource '
;============================================================
MENSAGEM_BLOG: 
       CALL   DELAY_15MS
       BCF    FLAG_RS  ;DEFINE ENVIO DE COMANDO (RS=0)
       MOVLW  0xC0
       MOVWF  DADO
       CALL   ENV_BYTE ;ENVIA COMANDO 'IR NA 2 LINHA'
       BSF    FLAG_RS ;DEFINE ENVIO DE DADOS (RS=1)
       MOVLW  ' '     ;ENVIA ESPAO ' '
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'B'     ;ENVIA 'B'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'l'     ;ENVIA 'l'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'o'     ;ENVIA 'o'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'g'     ;ENVIA 'g'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '     ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'P'    ;ENVIA   'P'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'i'     ;ENVIA  'i'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'c'     ;ENVIA  'c'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'S'     ;ENVIA  'S'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'o'    ;ENVIA   'o'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'u'     ;ENVIA   'u'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'r'     ;ENVIA   'r'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'c'     ;ENVIA   'c'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'e'     ;ENVIA   'e'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '     ;ENVIA ESPAO ' '
       MOVWF  DADO
       CALL   ENV_BYTE
       RETURN
;======================================================================
;ENVIA PARA 2 LINHA A MENSAGEM : 'READY' QUANDO ACABA
; A MEDIO COM CAPACITORES ACIMA DE 10mF
;======================================================================
MENSAGEM_READY: 
       BCF    FLAG_RS  ;DEFINE ENVIO DE COMANDO (RS=0)
       MOVLW  0xC0
       MOVWF  DADO
       CALL   ENV_BYTE ;ENVIA COMANDO 'IR NA 2 LINHA'
       BSF    FLAG_RS ;DEFINE ENVIO DE DADOS (RS=1)
#IFDEF ENGLISH
       MOVLW  ' '     ;ENVIA ESPAO ' '
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '     ;ENVIA ESPAO ' '
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '     ;ENVIA ESPAO ' '
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '     ;ENVIA ESPAO ' '
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '     ;ENVIA ESPAO ' '
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'R'     ;ENVIA ESPAO 'R'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'e'     ;ENVIA ESPAO 'e'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'a'     ;ENVIA ESPAO 'a'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'd'     ;ENVIA ESPAO 'd'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'y'     ;ENVIA ESPAO 'y'
       MOVWF  DADO
       CALL   ENV_BYTE
#ELSE
       MOVLW  ' '     ;ENVIA ESPAO ' '
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '     ;ENVIA ESPAO ' '
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '     ;ENVIA ESPAO ' '
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '     ;ENVIA ESPAO ' '
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'P'     ;ENVIA LETRA 'P'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'r'     ;ENVIA LETRA 'R'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'o'     ;ENVIA LETRA 'o'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'n'     ;ENVIA LETRA 'n'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  't'     ;ENVIA LETRA 't'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'o'     ;ENVIA LETRA 'o'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '     ;ENVIA ESPAO ' '
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '     ;ENVIA ESPAO ' '
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '     ;ENVIA ESPAO ' '
       MOVWF  DADO
       CALL   ENV_BYTE
#ENDIF
       RETURN
;=========================================================
;  ENVIA PARA LCD A MENSAGEM; 'WAIT' (AGUARDE)
;  PARA QUE O USURIO ESPERE  O FIM DA MEDIO 
;  (CAPACITORES ACIMA DE 10mF APENAS)
;=========================================================
MENSAGEM_WAIT:
 
       BCF    FLAG_RS ;DEFINE ENVIO DE COMANDO (RS=0)
       MOVLW  0xC0
       MOVWF  DADO
       CALL   ENV_BYTE ;ENVIA COMANDO DE IR PARA 2 LINHA
       BSF    FLAG_RS
#IFDEF  ENGLISH
       MOVLW  ' '     ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '     ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '     ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '     ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '     ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '     ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'W'     ;ENVIA 'W'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'a'     ;ENVIA 'a'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'i'     ;ENVIA 'i'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  't'     ;ENVIA 't'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '     ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
#ELSE
       MOVLW  ' '     ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '     ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '     ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '     ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '     ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'A'     ;ENVIA 'A'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'g'     ;ENVIA 'g'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'u'     ;ENVIA 'u'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'a'     ;ENVIA 'a'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'r'     ;ENVIA 'r'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'd'     ;ENVIA 'd'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'e'     ;ENVIA 'e'
       MOVWF  DADO
       CALL   ENV_BYTE
#ENDIF
       RETURN
;=========================================================
;  ENVIA PARA LCD UM AVISO DE BATERIA FRACA 'BAT LOW'
;=========================================================
MENSAGEM_BAT: 
       BCF    FLAG_RS
       MOVLW  0xC0
       MOVWF  DADO
       CALL   ENV_BYTE
       BSF    FLAG_RS

#IFDEF ENGLISH ;ENVIA MENSGEM 'LOW BAT'
       MOVLW  ' '   ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '   ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '   ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '   ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '   ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'L'    ;ENVIA 'T'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'O'   ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'W'   ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '   ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'B'   ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'A'   ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'T'    ;ENVIA 'B'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '    ;ENVIA 'A'
       MOVWF  DADO
       CALL   ENV_BYTE
#ELSE
       MOVLW  ' '   ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '   ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '   ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'T'    ;ENVIA 'T'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'r'   ;ENVIA  'r'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'o'   ;ENVIA  'o'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'q'   ;ENVIA  'q'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'u'   ;ENVIA  'u'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'e'   ;ENVIA  'e'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '    ;ENVIA ' '
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'B'    ;ENVIA 'B'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'a'    ;ENVIA 'a'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  't'   ;ENVIA  't'
       MOVWF  DADO
       CALL   ENV_BYTE
#ENDIF
       CALL   DELAY_1500MS ;DELAY DE VISUALISAO
       RETURN
;=========================================================
;  ENVIA PARA LCD UM AVISO DE ERRO
;  MOTIVADO POR SE COLOCAR UM CAPACITOR DE VALOR QUE
;  EXCEDE A CAPACIDADE DE MEDIO (ACIMA DE 10 F)
;=========================================================

MENSAGEM_ERRO:
       BCF    FLAG_RS ;DEFINE ENVIO DE COMANDO (RS=0)
       MOVLW  0xC0
       MOVWF  DADO
       CALL   ENV_BYTE ;ENVIA COMANDO DE IR PARA 2 LINHA
       BSF    FLAG_RS
#IFDEF  ENGLISH
       MOVLW  ' '     ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '     ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '     ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '     ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '     ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '     ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'E'     ;ENVIA 'E'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'r'     ;ENVIA 'r'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'r'     ;ENVIA 'r'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'o'     ;ENVIA 'o'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'r'     ;ENVIA 'r'
       MOVWF  DADO
       CALL   ENV_BYTE
#ELSE
       MOVLW  ' '     ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '     ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '     ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '     ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '     ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '     ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'E'     ;ENVIA 'E'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'r'     ;ENVIA 'r'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'r'     ;ENVIA 'r'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'o'     ;ENVIA 'o'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '     ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
#ENDIF
       RETURN
;=======================================================================
;MEDIO INICIAL PARA DETERMINAR CAPACITANCIAS PARASITAS EM ESCALA DE PF
;=======================================================================
MEDIR_CAP_REF: 
       CLRF   PONTO   ;AJUSTA ESCALA DE PF
       BANK1
       BSF   ESC_NF   ;ALTA IMPEDANCIA (NF)
       BANK0
       BSF   ESC_UF   ;BLOQUEIA TRANSISTOR PNP (ESCALA UF)
       BSF   ESC_MF   ;BLOQUEIA TRANSISTOR PNP (ESCALA MF)
       CALL   DISPARA_MEDICAO
LOOP_MD1_PF: 
       GOTO   $+1   
       BTFSS  OUT_555  ;TESTA OUT_555  RB4
       GOTO   SAI_DA_MEDICAO_REF     ;ENTRADA EM '0' Z=1
       INCF   RES1_L,F   ;ENTRADA EM '1' Z=0
       BTFSC  STATUS,Z
       INCF   RES1_H,F
       MOVF   RES1_L,W
       MOVWF  TEMP1_L
       MOVF   RES1_H,W
       MOVWF TEMP1_H
       MOVLW  0XF      ;0X270F = 9999
       MOVWF CONT0
       MOVLW  0x27
       MOVWF  CONT1
       CALL   COMPARA_REG   ;TESTA SE A MEDIO J CHEGOU A 9999
       BTFSC  STATUS,Z
       GOTO   LOOP_MD1_PF   ;MENOR - VOLTA A MEDIO NESTA ESCALA
       CALL   ESCALA_NF   ;MAIOR OU IGUAL - MUDA PARA PROXIMA ESCALA
       GOTO   LOOP_MD1_PF   ;VOLTA A MEDIO NESTA ESCALA

SAI_DA_MEDICAO_REF: 
       MOVF   RES1_L,W  ;OUT555=0 - SALVA MEDIO 'A' COMPLETADA
       MOVWF  RES1_L_SALVO
       MOVF   RES1_H,W
       MOVWF  RES1_H_SALVO
       RETURN  ;SAI DA ROTINA 'MEDIR_CAP_REF'
;==================================================================
;ROTINA DE MEDIO DE CAPACITORES
;=================================================================
MEDIR_CAPACITOR: 
       MOVF   RES1_L_SALVO,W
       MOVWF  RES1_L    ;MEDIO ANTERIOR L
       MOVF   RES1_H_SALVO,W
       MOVWF  RES1_H    ;MEDIO ANTERIOR H
       MOVLW  'p'       ;CARREGA PREFIXO 'p'
       MOVWF  PREFIXO
       CLRF   PONTO    ;INICIA EM ESCALA DE PF
       BANK1
       BSF   ESC_NF    ; (DE 0 PF A 9,99 NANOFARADS)
       BANK0
       BSF   ESC_UF    ;BLOQUEIA TRANSISTOR PNP DA ESCALA DE UF
       BSF   ESC_MF    ;BLOQUEIA TRANSISTOR PNP DA ESCALA DE MF
       CALL   DISPARA_MEDICAO ;INICIA PULSO POSITIVO NO PINO 3 DO 555
LOOP_MD2_PF: 
       GOTO   $+1
       BTFSS  OUT_555  ;TESTA OUT_555  RB4
       GOTO   SAI_DA_MEDICAO_PF    ; '0' FIM DO PULSO NO PINO 3 DO 555
AINDA_MEDINDO:
       INCF   RES_L,F  ;INCREMENTA RESULTADO L
       BTFSC  STATUS,Z
       INCF   RES_H,F  ;INCREMENTA RESULTADO H
       MOVF   RES_L,W
       MOVWF  TEMP1_L
       MOVF   RES_H,W
       MOVWF TEMP1_H
       MOVLW  0XF     ;COMPARA COM 9999 (0X270F)
       MOVWF CONT0
       MOVLW  0X27
       MOVWF  CONT1
       CALL   COMPARA_REG
       BTFSC  STATUS,Z
       GOTO   LOOP_MD2_PF
       CALL   ESCALA_NF    ;VAI PARA ESCALA DE NANOFARADS
       GOTO   LOOP_MD2_PF
;============================================================
;OBTEM VALOR CORRETO SUBTRAINDO O VALOR DE REFERENCIA 
;PARA ELIMINAR VALOR RELATIVO AS CAPACITANCIAS PARASITAS
; (AUTOZERO)
;============================================================
SAI_DA_MEDICAO_PF: 
       MOVF   RES_L,W   ;SUBTRAI MEDIO 'B' DE 'A'
       MOVWF  TEMP1_L
       MOVF   RES_H,W
       MOVWF TEMP1_H
       MOVF   RES1_L,W
       SUBWF  TEMP1_L,F
       BTFSS  STATUS,C
       DECF  TEMP1_H,F
       MOVF   RES1_H,W
       SUBWF TEMP1_H,F
       MOVF   TEMP1_L,W
       MOVWF  RES_L    ;SALVA RESULTADO L
       MOVF  TEMP1_H,W
       MOVWF  RES_H    ;SALVA RESULTADO H
;===========================================================
;COMPARA RESULTADO COM VALOR MXIMO = 9999
TESTA_VALOR_MAX: 
       MOVF   RES_L,W
       MOVWF  TEMP1_L
       MOVF   RES_H,W
       MOVWF TEMP1_H
       MOVLW  0XF      ;0X270F = 9999
       MOVWF CONT0
       MOVLW  0X27
       MOVWF  CONT1
       CALL   COMPARA
       BTFSC  STATUS,Z
       GOTO   VALOR_MENOR_MAX
       CLRF  RES_L  ;CANCELA MEDIO
       CLRF  RES_H  ;ZERANDO RESULTADO
       MOVF  N_LEITURAS,W  ;SE NA ULTIMA ESCALA (MF)
       SUBLW 4             ;TESTA SE CAPACITOR
       BTFSC STATUS,Z      ;ULTRAPASSOU VALOR MAXIMO
       CALL  MENSAGEM_ERRO ;ULTRAPASSOU CAPACIDADE 
      
VALOR_MENOR_MAX: 
       MOVF   RES_L,W
       MOVWF  TEMP1_L
       MOVF   RES_H,W
       MOVWF TEMP1_H
       MOVLW 0XE8      ;COMPARA COM  1000  (0X3E8) 
       MOVWF CONT0
       MOVLW  3
       MOVWF  CONT1
       CALL   COMPARA_REG
       BTFSC  STATUS,Z
       GOTO   BAT_TEST
       MOVF   RES_L,W
       MOVWF  TEMP1_L
       MOVF   RES_H,W
       MOVWF TEMP1_H
       MOVLW  0xA     ;DIVIDE POR 10
       MOVWF CONT0
       MOVLW  0
       MOVWF  CONT1
       CALL   FAZ_DIVISAO
       MOVF   TEMP1_L,W
       MOVWF  RES_L
       MOVF  TEMP1_H,W
       MOVWF  RES_H
       MOVLW  'n'      ;PREFIXO 'n'
       MOVWF  PREFIXO    ;SALVA NO REGISTRADOR DE PREFIXO
       MOVLW  1
       MOVWF  PONTO
;=========================================================
;TESTA TENSO DA BATERIA 
;(SE MENOR QUE 6 VOLTS ENVIA MENSAGEM: 'BAT LOW'
;=========================================================
BAT_TEST:
       BTFSS  V_BAT
       CALL   MENSAGEM_BAT  ;MOSTRA 'BAT LOW' NA 2 LINHA
       RETURN
;=========================================================      
;ESCALA DE NANOFARADS
; (DE 10 NANOFARADS A 9,99 MICROFARADS)
;=========================================================
ESCALA_NF: 
       MOVLW  'n'    ;CARREGA PREFIXO 'n'
       MOVWF  PREFIXO
       CLRF   PONTO
       CLRF   RES1_L
       CLRF   RES1_H
       MOVLW  1
       MOVWF  ESCALA
       BANK1
       BCF   ESC_NF    ;AJUSTA ESCALA DE NF
       BANK0
       BSF   ESC_NF    ;SADA EM '1'
       BSF   ESC_UF    ;BLOQUEIA ESCALA DE UF
       BSF   ESC_MF   ;BLOQUEIA TRANSISTOR PNP (ESCALA MF)
       CALL   DISPARA_MEDICAO
MEDINDO_NF: 
       GOTO   $+1
       BTFSS  OUT_555  ;TESTA OUT_555  RB4
       GOTO   FIM_MEDICAO_NF    ;FIM DA MEDIO
       INCF   RES_L,F  ;CONTINUA A MEDIO
       BTFSC  STATUS,Z
       INCF   RES_H,F  ;INCREMENTA RESULTADOS
       MOVF   RES_L,W
       MOVWF  TEMP1_L
       MOVF   RES_H,W
       MOVWF TEMP1_H
       MOVLW  0XF       ;0X270F=9999
       MOVWF CONT0
       MOVLW  0X27
       MOVWF  CONT1
       CALL   COMPARA_REG
       BTFSC  STATUS,Z
       GOTO   MEDINDO_NF
       CALL   ESCALA_UF  ;VAI PARA ESCALA DE MICROFARADS (uF)
       GOTO   MEDINDO_NF

FIM_MEDICAO_NF: 
       MOVF   RES_L,W  ;TRANSFERE DE REGISTRADORES
       MOVWF  TEMP1_L  ;PARA EFETUAR A COMPARAO
       MOVF   RES_H,W
       MOVWF TEMP1_H
       MOVLW  0XE8      ;0X3E8 = 1000
       MOVWF CONT0
       MOVLW  3
       MOVWF  CONT1
       CALL   COMPARA_REG  ;FAZ COMPARAO COM 1000
       BTFSC  STATUS,Z
       RETURN ;MENOR QUE 1000
       MOVF   RES_L,W  ;MAIOR QUE 1000
       MOVWF  TEMP1_L
       MOVF   RES_H,W
       MOVWF TEMP1_H
       MOVLW  0xA      ;DIVIDE POR 10
       MOVWF CONT0
       MOVLW  0
       MOVWF  CONT1
       CALL   FAZ_DIVISAO ;ROTINA DE DIVISO
       MOVF   TEMP1_L,W
       MOVWF  RES_L       ;SALVA RESULTADO L
       MOVF  TEMP1_H,W
       MOVWF  RES_H       ;SALVA RESULTADO H
       MOVLW  'u'       ;CARREGA PREFIXO 'u'
       MOVWF  PREFIXO
       MOVLW  1         ;COLOCA PONTO
       MOVWF  PONTO
       RETURN
;===========================================================
; ROTINA DE TESTE NA ESCALA DE MICROFARADS
; (CAPACITORES AT 9999 MICROFARADS)
;==========================================================
ESCALA_UF: 
       MOVLW  1
       MOVWF  ESCALA ;ESCALA EM '1'
       MOVLW  'u'    ;PREFIXO CARREGADO COM 'u'
       MOVWF  PREFIXO
       CLRF   PONTO  ;INICIA SEM PONTO
       CLRF   RES1_L ;ZERA REGISTRADORES
       CLRF   RES1_H
       BANK1
       BSF   ESC_NF  ;PINO EM ALTA IMPEDANCIA
       BANK0
       BCF   ESC_UF  ;ACIONA TRANSISTOR DA ESCALA DE uF
       BSF   ESC_MF   ;BLOQUEIA TRANSISTOR PNP (ESCALA MF)
       CLRF   RES_L  ;ZERA REGISTRADORES
       CLRF   RES_H
       CALL   RESETA_555 ;RESETA 555
       BSF   DISP_555
       CALL   DELAY_1500MS  ;DELAY DE 1,5 SEGUNDOS
       BCF   DISP_555 ;DISPARA 555
       CALL   DELAY_DISP ;DELAY MNIMO
       BSF   DISP_555 ;SETA PINO
MEDINDO_UF: 
       GOTO   $+1
       BTFSS  OUT_555  ;TESTA OUT_555  RB4
       GOTO   FIM_MEDICAO_UF ;VOLTA PARA TESTAR SADA DO 555
       
;============================================================
;ROTINA DE DELAY DE 190 uSEG  (188.0)
;============================================================
       CLRF   DELAY_2
DELAY_190US: 
       MOVF   DELAY_2,W
       SUBLW  0x5C
       CLRW
       BTFSC  STATUS,C
       ADDLW  0xFF
       BTFSC  STATUS,Z
       GOTO   PULA3
       INCF   DELAY_2,F
       GOTO   DELAY_190US
;=============================================================
PULA3: INCF   RES_L,F   ;INCREMENTA RESULTADO L
       BTFSC  STATUS,Z
       INCF   RES_H,F   ;INCREMENTA RESULTADO H
       MOVF   RES_L,W
       MOVWF  TEMP1_L
       MOVF   RES_H,W
       MOVWF TEMP1_H
       MOVLW  0XF       ;COMPARA COM 9999 (0X270F)
       MOVWF CONT0
       MOVLW  0x27
       MOVWF  CONT1
       CALL   COMPARA_REG ;ROTINA DE COMPARAO
       BTFSC  STATUS,Z
       GOTO   MEDINDO_UF  ;MENOR QUE 9999 E CONTINUA MEDINDO
       CALL   ESCALA_MF  ;MAIOR - VAI PARA ESCALA DE MILIFARADS
       GOTO   MEDINDO_UF ;VOLTA PARA TESTAR SAIDA DO 555
FIM_MEDICAO_UF: 
       MOVF   RES_L,W
       MOVWF  TEMP1_L ;TRANSFERE DE REGISTRADOR PARA A COMPARAO
       MOVF   RES_H,W
       MOVWF TEMP1_H
       MOVLW  0XE8    ;COMPARA COM 1000
       MOVWF CONT0
       MOVLW  3
       MOVWF  CONT1
       CALL   COMPARA_REG ;ROTINA DE COMPARAO
       BTFSC  STATUS,Z
       RETURN ;MENOR
       MOVF   RES_L,W ;MAIOR QUE 1000
       MOVWF  TEMP1_L
       MOVF   RES_H,W
       MOVWF TEMP1_H
       MOVLW  0xA     ;DIVIDE POR 10
       MOVWF CONT0
       MOVLW  0
       MOVWF  CONT1
       CALL   FAZ_DIVISAO ;ROTINA DE DIVISO
       MOVF   TEMP1_L,W
       MOVWF  RES_L
       MOVF  TEMP1_H,W
       MOVWF  RES_H
       MOVLW  'm'     ;CARREGA PREFIXO 'm'
       MOVWF  PREFIXO
       MOVLW  1       ;COLOCA PONTO
       MOVWF  PONTO
       RETURN

;=====================================================
;ROTINA DE MEDIO NA ESCALA DE MILIFARADS
; (ACIMA DE 10 MILIFARADS)
;=====================================================
ESCALA_MF: 
       INCF   N_LEITURAS,F ;INCREMENTA NMERO DE LEITURAS
       MOVLW  1
       MOVWF  ESCALA
       MOVLW  'm'     ;CARREGA PREFIXO 'm'
       MOVWF  PREFIXO
       CLRF   PONTO   ;SEM PONTO
       BANK1
       BSF   ESC_NF  ;PINO EM ALTA IMPEDANCIA
       BANK0
       BSF   ESC_UF  ;BLOQUEIA TRANSISTOR DA ESCALA DE uF
       BCF   ESC_MF   ;ACIONA TRANSISTOR PNP (ESCALA MF)
       CLRF   RES1_L  ;LIMPA REGISTRADORES
       CLRF   RES1_H
       CLRF   RES_L
       CLRF   RES_H
       CALL   RESETA_555
       CALL   MENSAGEM_WAIT ;MENSAGEM 'WAIT' NO LCD
       BSF    DISP_555     ;PINO 2 EM '1'
       CALL   DELAY_60SEG  ;DELAY P/DESCARGA DO CAPACITOR
       BCF    DISP_555     ;PINO 2 EM '0'
       CALL   DELAY_DISP   ;DELAY MNIMO
       BSF    DISP_555     ;PINO 2 EM '1'
MEDINDO_MF: 
       GOTO   $+1
       BTFSS  OUT_555  ;TESTA OUT_555  RB4
       GOTO   FIM_MEDICAO_MF ;PINO 3 DO 555 EM '0'
       CALL   DELAY_100MS  ; PINO 3 DO 555 EM '1' - DELAY 100MS  
       INCF   RES_L,F      ;INCREMENTA RESULTADO
       BTFSC  STATUS,Z
       INCF   RES_H,F
       GOTO   MEDINDO_MF   ;VAI TESTAR SAIDA DO 555
FIM_MEDICAO_MF: 
       MOVF   RES_L,W  ;TRANSFERE DE REGISTRADORES P/ COMPARAO
       MOVWF  TEMP1_L
       MOVF   RES_H,W
       MOVWF TEMP1_H
       MOVLW  0XE8    ;COMPARA COM 1000
       MOVWF CONT0
       MOVLW  3
       MOVWF  CONT1
       CALL   COMPARA_REG  ;ROTINA DE COMPARAO
       BTFSC  STATUS,Z
       GOTO   TESTA_N_LEITURAS ;MENOR QUE 1000
       ;SE MAIOR QUE 1000, PASSA PARA ESCALA DE FARADS
       MOVF   RES_L,W
       MOVWF  TEMP1_L
       MOVF   RES_H,W
       MOVWF TEMP1_H
       MOVLW  0xA    ;DIVIDE POR 10
       MOVWF CONT0
       MOVLW  0
       MOVWF  CONT1
       CALL   FAZ_DIVISAO ;ROTINA DE DIVISO
       MOVF   TEMP1_L,W
       MOVWF  RES_L
       MOVF  TEMP1_H,W
       MOVWF  RES_H
       MOVLW  ' '     ;APAGA PREFIXO (SOMENTE FARADS)
       MOVWF  PREFIXO
       MOVLW  1       ;COLOCA PONTO (X.XX F)
       MOVWF  PONTO
TESTA_N_LEITURAS:
       MOVF   N_LEITURAS,W
       SUBLW  4             ;APS 4 LEITURAS MOSTRA 'READY'
       BTFSS  STATUS,Z
       RETURN ;SAI DA ROTINA PARA CONTINUAR A MEDIO NESTA ESCALA
       CALL   MENSAGEM_READY ;MENSAGEM 'READY' INDICANDO FIM DA MEDIO
       GOTO   $    ;LOOP ETERNO - TEM QUE APERTAR RESET OU DESLIGAR ALIMENTAO
;================================================================================       
; ROTINAS DE RESET E DISPARO DO 555 MOSFET
;================================================================================
RESETA_555: ;RESETA 555
       BSF   RESET_555  ;PINO 4 DO 555 EM '1'
       BCF   RESET_555  ;PINO 4 DO 555 EM '0'
       CALL  DELAY_1MS  ;DELAY DE 1MS
       BSF   RESET_555 ;PINO 4 DO 555 EM '1'
       RETURN
;===============================================================================
DISPARA_MEDICAO: ;INICIA TEMPORIZAO NO 555
       CLRF   RES_L
       CLRF   RES_H
       CALL   RESETA_555 ;PINO 4 DO 555 FICA EM '0' E DEPOIS '1'
       BSF   DISP_555   ; PINO 2 DO 555 EM '1'
       CALL  DELAY_100MS ;DELAY 100 MS
       BCF   DISP_555   ;PULSA PINO 2 INICIANDO TEMPO DO 555 (PINO 3 =1)
       CALL   DELAY_DISP     ;DELAY DE 6,8 USEG
       BSF   DISP_555   ;RETIRA PULSO
       RETURN
;===============================================================================
;ROTINA DE TRATAMENTO DO RESULTADO DA MEDIO E ENVIO
; PARA O LCD
;===============================================================================
SHOW_RESULTADO:          
       BCF    FLAG_RS    ;ENVIA COMANDO (RS=0)
       MOVLW  1          ;COMANDO DE LIMPEZA DE TELA DO LCD
       MOVWF  DADO
       CALL   ENV_BYTE   ;ENVIA COMANDO DE LIMPEZA DA TELA
ENVIA_ESPACO_C_:
       BSF    FLAG_RS    ;ENVIA DADO (RS=1)
       CLRF   MILHARES
       CLRF   CENTENAS
       CLRF   DEZENAS
       MOVLW  ' '       ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '       ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  ' '       ;ENVIA ESPAO
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  'C'       ;ENVIA LETRA 'C'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  '='       ;ENVIA CARACTER '='
       MOVWF  DADO
       CALL   ENV_BYTE

OBTEM_MILHAR: 
       MOVF   RES_L,W   ;OBTEM DGITO DO MILHAR (QUANDO TEM PONTO)
       MOVWF  TEMP1_L
       MOVF   RES_H,W
       MOVWF  TEMP1_H
       MOVLW  0XE8      ;COMPARA RESULTADO COM 1000 (0X3E8)
       MOVWF  CONT0
       MOVLW  3
       MOVWF  CONT1
       CALL   COMPARA_REG ;ROTINA DE COMPARAO
       BTFSC  STATUS,Z
       GOTO   ENV_MILHAR_LCD ;MENOR QUE 1000 - OK, J TEMOS A MILHAR!
MAIOR_IGUAL_1000:            ;MAIOR - CONTINUA A ROTINA!   
       INCF   MILHARES,F ;INCREMENTA REGISTRADOR DE MILHAR
       MOVF   RES_L,W
       MOVWF  TEMP1_L    ;TRANSFERE PARA A SUBTRAO
       MOVF   RES_H,W
       MOVWF TEMP1_H
       MOVLW  0XE8
       SUBWF  TEMP1_L,F   ;SUBTRAI 1000
       BTFSS  STATUS,C
       DECF  TEMP1_H,F
       MOVLW  3
       SUBWF TEMP1_H,F
       MOVF   TEMP1_L,W
       MOVWF  RES_L      ;SALVA RESULTADO L
       MOVF  TEMP1_H,W
       MOVWF  RES_H      ;SALVA RESULTADO H
       GOTO   OBTEM_MILHAR  ;VOLTA AO LOOP

ENV_MILHAR_LCD: 
       MOVF   MILHARES,W ;MOVE PARA 'W'
       BTFSS  STATUS,Z   ;TESTA SE ZERO
       GOTO   MILHAR_N_ZERO ;DGITO DIFERENTE DE '0'
APAGAR_MILHAR:         ; ZERO
       MOVLW  ' '      ;ENVIA ESPAO (CASA MILHAR=0)
       MOVWF  DADO
       GOTO   ENVIAR_1 ;VAI PARA ROTINA DE ENVIO P/ LCD
MILHAR_N_ZERO: 
       MOVF   MILHARES,W ;MILHAR DIFERENTE DE '0'-ENVIA!
       ADDLW  0x30     ;SOMA 0X30 PARA CONVERTER ASCII
       MOVWF  DADO
ENVIAR_1: 
       CALL   ENV_BYTE ;ENVIA DIGITO PARA O LCD

OBTEM_CENTENA:         ;OBTEM DGITO DA CENTENA
       MOVF   RES_L,W
       MOVWF  TEMP1_L
       MOVF   RES_H,W
       MOVWF TEMP1_H
       MOVLW  0x64     ;COMPARA COM 100
       MOVWF CONT0
       MOVLW  0
       MOVWF  CONT1
       CALL   COMPARA_REG ;COMPARA SE MAIOR QUE 100
       BTFSC  STATUS,Z
       GOTO   ENVIA_CENTENA_LCD ;MENOR - OK, TEMOS A CENTENA!
MAIOR_QUE_100:                  ;MAIOR - DE VOLTA A ROTINA!
       INCF   CENTENAS,F ;INCREMENTA 
       MOVF   RES_L,W
       MOVWF  TEMP1_L
       MOVF   RES_H,W
       MOVWF TEMP1_H
       MOVLW  0x64       ;100  (0X64)
       SUBWF  TEMP1_L,F  ;SUBTRAI 100
       BTFSS  STATUS,C
       DECF  TEMP1_H,F
       MOVLW  0
       SUBWF TEMP1_H,F
       MOVF   TEMP1_L,W
       MOVWF  RES_L     ;SALVA RESULTADO L
       MOVF  TEMP1_H,W
       MOVWF  RES_H     ;SALVA RESULTADO H
       GOTO   OBTEM_CENTENA ;VOLTA AO LOOP
ENVIA_CENTENA_LCD:
       MOVF   MILHARES,W  ;DETERMINA SE TEM ZEROS A ESQUERDA
       IORWF  CENTENAS,W
       BTFSS  STATUS,Z
       GOTO   CENTENA_N_ZERO ; MILHAR E CENTENA DIFERENTES DE '0'
APAGAR_CENTENA:
       MOVLW  ' '      ;ENVIA ESPAO (SE MILHAR/CENTENA FOREM '0')
       MOVWF  DADO
       GOTO   ENVIAR_2
CENTENA_N_ZERO: 
       MOVF   CENTENAS,W
       ADDLW  0x30     ;SOMA 0X30 PARA CONVERTER ASCII
       MOVWF  DADO
ENVIAR_2: 
       CALL   ENV_BYTE ;ENVIA DIGITO PARA LCD
       MOVF   PONTO,W  ;TESTA SE DEVE ENVIAR PONTO/VIRGULA
       BTFSC  STATUS,Z
       GOTO   OBTEM_DEZENA ;NO ENVIA E VAI OBTER A DEZENA
ENVIAR_PONTO: ;ENVIA PONTO/VIRGULA - CONFORME OPO DO USURIO
       #IFDEF VIRGULA
       MOVLW  ','     ;ENVIA VIRGULA
       #ELSE
       MOVLW  '.'     ;ENVIA PONTO
       #ENDIF
       MOVWF  DADO
       CALL   ENV_BYTE  ;ENVIA PARA O LCD

OBTEM_DEZENA:   ;OBTEM O DGITO DAS DEZENAS
       MOVF   RES_L,W
       MOVWF  TEMP1_L
       MOVF   RES_H,W
       MOVWF TEMP1_H
       MOVLW  0xA     ;COMPARA COM 10
       MOVWF CONT0
       MOVLW  0
       MOVWF  CONT1
       CALL   COMPARA_REG
       BTFSC  STATUS,Z
       GOTO   ENVIAR_DEZENA_LCD ;OK, TEMOS A DEZENA!
       INCF   DEZENAS,F ;AINDA PRECISAMOS CONTINUAR!
       MOVF   RES_L,W
       MOVWF  TEMP1_L
       MOVF   RES_H,W
       MOVWF TEMP1_H
       MOVLW  0xA      ;SUBTRAI 10 DO RESULTADO
       SUBWF  TEMP1_L,F
       BTFSS  STATUS,C
       DECF  TEMP1_H,F
       MOVLW  0
       SUBWF TEMP1_H,F
       MOVF   TEMP1_L,W
       MOVWF  RES_L    ;SALVA RESULTADO L
       MOVF  TEMP1_H,W
       MOVWF  RES_H    ;SALVA RESULTADO H
       GOTO   OBTEM_DEZENA ;VOLTA AO LOOP
ENVIAR_DEZENA_LCD: 
       MOVF   MILHARES,W  ;DETERMINA SE TEM ZEROS A ESQUERDA
       IORWF  CENTENAS,W
       IORWF  DEZENAS,W
       BTFSS  STATUS,Z
       GOTO   DEZENAS_N_ZERO ;VAI ENVIAR O DGITO DA DEZENA
APAGAR_DEZENAS:       ;NO ENVIA, MAS APAGA A CASA A ESQUERDA
       MOVLW  ' '     ;ENVIA ESPAO (ELIMINA ZEROS A ESQUERDA)
       MOVWF  DADO
       GOTO   ENVIAR_3
DEZENAS_N_ZERO: 
       MOVF   DEZENAS,W ;ENVIA DEZENAS
       ADDLW  0x30   ;SOMA 0X30 PARA CONVERTER ASCII
       MOVWF  DADO
ENVIAR_3: 
       CALL   ENV_BYTE ;ENVIA PARA O LCD
       MOVF   RES_L,W  ;ENVIA UNIDADES
       ADDLW  0x30   ;SOMA 0X30 PARA CONVERTER ASCII
       MOVWF  DADO
       CALL   ENV_BYTE ;ENVIA PARA O LCD
       MOVLW  ' '     ;ENVIA ESPAO ANTES DO PREFIXO
       MOVWF  DADO
       CALL   ENV_BYTE ;ENVIA PARA O LCD
       MOVF   PREFIXO,W ;ENVIA PREFIXO (u,n,p,m)
       MOVWF  DADO
       CALL   ENV_BYTE ;ENVIA PARA O LCD
       MOVLW  'F'      ;ENVIA LETRA 'F'
       MOVWF  DADO
       CALL   ENV_BYTE ;ENVIA PARA O LCD
       RETURN
;====================================================
; ROTINA DE INICIALIZAO DOS PORTS 
PORTS_INIT:
       BANK0
       MOVLW  0X07
       MOVWF  CMCON  ; DESLIGA COMPARADORES ANALOGICOS
       BANK1  ;VAI NO TRISA/TRISB    
       BCF   LCD_RS ;AJUSTA DIREO DO PINOS (IN/OUT)
       BCF   LCD_E  ;COMO SADAS
       BCF   LCD_D4
       BCF   LCD_D5
       BCF   LCD_D6
       BCF   LCD_D7
       BCF   ESC_UF
       BCF   ESC_NF
       BCF   ESC_MF
       BCF   RESET_555
       BCF   DISP_555
       BSF   OUT_555
       BSF    V_BAT ;EXCETO ESTA COMO ENTRADA
       BANK0
       CLRF  PORTA  ;DESLIGA PORTS
       CLRF  PORTB
       RETURN

;***************************************************************************
;            ROTINAS DE LCD
;***************************************************************************
;===========================================================================
; ROTINA DE INICIALIZAO DO LCD PARA USO DE 4 LINHAS DE DADOS
;===========================================================================
LCD_INIT: 
       CALL   DELAY_15MS   ;DELAY 15 MS (14.651,8 uS OR=2,2)
       ;SEQUENCIA PARA USAR  4 FIOS NO LCD (DB4 A DB7)
       MOVLW  0x30       ;ENVIA '30H'
       MOVWF  DADO
       CALL   ENV_NIBBLE
       MOVLW  0x30       ;ENVIA '30H'
       MOVWF  DADO
       CALL   ENV_NIBBLE
       MOVLW  0x30       ;ENVIA '30H'
       MOVWF  DADO
       CALL   ENV_NIBBLE
       MOVLW  0x20       ;ENVIA '20H'
       MOVWF  DADO
       CALL   ENV_NIBBLE
       BCF    FLAG_RS
       MOVLW  0x28       ;ENVIA '28H'
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  0x18       ;ENVIA '18H' (DESLOCA MENSAGEM)
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  0XC        ;ENVIA '0CH' (DESATIVA CURSOR)
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  4          ;ENVIA '04H' (MODO)
       MOVWF  DADO
       CALL   ENV_BYTE
       MOVLW  1         ;ENVIA 01H   (APAGA LCD)
       MOVWF  DADO
       CALL   ENV_BYTE
       RETURN
;===================================================
; ENVIA NIBBLE PARA LCD NA INICIALIZAO
ENV_NIBBLE: 
       CALL   DELAY_10MS   ;DELAY DE 10 MS
       BCF   LCD_RS      ;RESETA RS
       CALL   ENV_LCD    ;ENVIA COMANDO PARA LCD
       RETURN
;--------------------------------------
;ENVIA UM BYTE DE COMANDO OU DADO PARA LCD
ENV_BYTE: 
       BTFSC  FLAG_RS  ;TESTA FLAG DE COMANDO OU DADO
       GOTO   ENV_001
       BCF   LCD_RS    ;ENVIAR COMANDO
       CALL  DELAY_4MS ;DELAY PARA COMANDO
       GOTO   ENV_002
ENV_001:
       BSF   LCD_RS    ;ENVIAR DADO
       CALL  DELAY_1MS ;DELAY PARA DADOS
ENV_002: 
       CALL   ENV_LCD
       SWAPF  DADO,F  ;SWAP NIBBLES
       CALL   ENV_LCD  ;ENVIA O OUTRO NIBBLE
       RETURN
;=======================================
;ENVIA BYTE DE DADOS OU COMANDOS PARA O LCD
ENV_LCD: 
       BSF   LCD_D4  ;COLOCA NIBBLE NO PINOS DB4 A DB7
       BTFSS  DADO,4
       BCF   LCD_D4
       BSF   LCD_D5
       BTFSS  DADO,5
       BCF   LCD_D5
       BSF   LCD_D6
       BTFSS  DADO,6
       BCF   LCD_D6
       BSF   LCD_D7
       BTFSS  DADO,7
       BCF   LCD_D7
       BSF   LCD_E   ;D UM 'CLOCK' NA LINHA 'LCD_E'
       BCF   LCD_E
       RETURN
;====================================================
;ROTINAS DE DIVISO
;====================================================
FAZ_DIVISAO: 
       CALL   ROT_DIV    ;CHAMA ROTINA DE DIVISO
       MOVF   TEMP1_L0,W ;SALVA RESULTADO
       MOVWF  TEMP1_L
       MOVF   TEMP1_L1,W
       MOVWF TEMP1_H
       RETURN
;====================================================
;ROTINA DE DIVISO
;SENDO TEMP1_L/H COM VALOR A DIVIDIR
; E CONT_0/1 O DIVISOR
;RESULTADO EM TEMP1_L0(L)/TEMP_L1(H)
;===================================================

ROT_DIV: 
       CLRF   TEMP1_L0
       CLRF   TEMP1_L1
       MOVF  CONT0,W
       IORWF  CONT1,W 
       BTFSC  STATUS,Z
       RETURN    ;NO DIVIDE POR '0'
       MOVLW  1
       MOVWF  DIV_0
       CLRF   DIV_1
ROTATE1: 
       BTFSC  CONT1,7  ;TESTA BIT 7 (PARA ALINHAR)
       GOTO   LOOP_DIV1 ;VAI EFETIVAR A DIVISO
       BCF    STATUS,C
       RLF    DIV_0,F  ;DESLOCA PARA ESQUERDA
       RLF    DIV_1,F
       RLF   CONT0,F
       RLF    CONT1,F
       GOTO   ROTATE1  ;VOLTA A ROTACIONAR
LOOP_DIV1: 
       MOVF   CONT1,W
       SUBWF TEMP1_H,W
       BTFSC  STATUS,Z ;TESTA SE ZERO
       GOTO   PULA_DIV2
       BTFSS  STATUS,C ;TESTA BORROW
       GOTO   PULA_DIV3
       GOTO   PULA_DIV4
PULA_DIV2: 
       MOVF  CONT0,W
       SUBWF  TEMP1_L,W
       BTFSS  STATUS,C
       GOTO   PULA_DIV3
PULA_DIV4: 
       MOVF  CONT0,W
       SUBWF  TEMP1_L,F
       BTFSS  STATUS,C
       DECF  TEMP1_H,F
       MOVF   CONT1,W
       SUBWF TEMP1_H,F
       MOVF   DIV_0,W
       ADDWF  TEMP1_L0,F
       BTFSC  STATUS,C
       INCF   TEMP1_L1,F
       MOVF   DIV_1,W
       ADDWF  TEMP1_L1,F
PULA_DIV3:
       BCF    STATUS,C
       RRF    CONT1,F  ;DESLOCA PARA DIREITA
       RRF   CONT0,F
       RRF    DIV_1,F
       RRF    DIV_0,F
       BTFSS  STATUS,C
       GOTO   LOOP_DIV1 ;VOLTA AO LOOP1
       RETURN


;======================================
;SUBTRAI PAR TEMP0/1 DE CONT1/E
; SE IGUAL OU MAIOR RETORNA 0XFF
; SE MENOR RETORNA 0X00
COMPARA: 
       MOVF  TEMP1_H,W
       SUBWF  CONT1,W  ;SUBTRAI H
       BTFSS  STATUS,Z
       GOTO   RESULTADO_N_Z
       MOVF   TEMP1_L,W
       SUBWF CONT0,W   ;SUBTRAI L
RESULTADO_N_Z:  ; MENOR
       BTFSS  STATUS,C
       GOTO   E_MAIOR_OU_IGUAL
       CLRF   TEMP1_L   ;ZERA REGISTRADORES
       CLRF  TEMP1_H
       RETURN
E_MAIOR_OU_IGUAL:     ; MAIOR
       IORLW  0xFF
       MOVWF  TEMP1_L ;RETORNA 0XFF
       MOVWF TEMP1_H
       RETURN


;================================
;SUBTRAI PAR CONT1/E DE TEMP 0/1 
; SE IGUAL OU MAIOR RETORNA 0XFF
; SE MENOR RETORNA 0X00
COMPARA_REG: 
       MOVF   CONT1,W
       SUBWF TEMP1_H,W
       BTFSS  STATUS,Z
       GOTO   RESULTADO_NZ_1
       MOVF  CONT0,W
       SUBWF  TEMP1_L,W
       BTFSC  STATUS,Z
       GOTO   FOI_IGUAL_MAIOR
RESULTADO_NZ_1: 
       BTFSC  STATUS,C
       GOTO   FOI_IGUAL_MAIOR
       CLRF   TEMP1_L
       CLRF  TEMP1_H
       RETURN
FOI_IGUAL_MAIOR: 
       IORLW  0xFF
       MOVWF  TEMP1_L
       MOVWF TEMP1_H
       RETURN
;*************************************************************
;       ROTINAS DE DELAYS
;*************************************************************
;==================================================
; DELAY CURTO (6,6 US) P/ DISPARO DO 555
DELAY_DISP: 
       CLRF   DELAY_0
L01D2: MOVF   DELAY_0,W
       SUBLW  1
       CLRW
       BTFSC  STATUS,C
       ADDLW  0xFF
       BTFSC  STATUS,Z
       GOTO   SAI_1 ;L01DB
       INCF   DELAY_0,F
       GOTO   L01D2
SAI_1: RETURN
;==============================================
;ROTINA DE DELAY DE 60 SEG
DELAY_60SEG:
       MOVLW  0x60      ;DELAY DE 58,59 SEG
       MOVWF  TEMP1_L   ;(58596190 uS)
       MOVLW  0XEA
       MOVWF  TEMP1_H
       GOTO   DELAY_MS
;===============================================
;ROTINA DE DELAY DE 1,5 SEG
DELAY_1500MS: ;1.464.907 uSEG   
       MOVLW  0XDC     ;DELAY  1,5 S
       MOVWF  TEMP1_L
       MOVLW  0X5
       MOVWF TEMP1_H
       GOTO  DELAY_MS

;==============================================
;ROTINA DE DELAY DE 1 SEG
DELAY_1SEG:
       MOVLW  0xE8      
       MOVWF  TEMP1_L
       MOVLW  3
       MOVWF TEMP1_H
       GOTO   DELAY_MS  
;============================================
;ROTINA DE DELAY DE 190 MS
DELAY_190MS: ;(189.463,4 uS  OR=,2) 
       MOVLW  0xC2       ;DELAY 190 MS
       MOVWF  TEMP1_L    
       CLRF TEMP1_H
       GOTO   DELAY_MS
;============================================
;ROTINA DE DELAY DE 100 MS
DELAY_100MS:  ;(97.662,8 uS)
       MOVLW  0x64      
       MOVWF  TEMP1_L
       MOVLW  0
       MOVWF TEMP1_H
       GOTO   DELAY_MS  
;=============================================
;ROTINA DE DELAY DE 15MS
DELAY_15MS:
       MOVLW  0XF
       MOVWF  TEMP1_L
       MOVLW  0
       MOVWF TEMP1_H
       GOTO   DELAY_MS   
;=============================================
;ROTINA DE DELAY DE 10 MS
DELAY_10MS:
       MOVLW  0xA       ;DELAY DE 10 MS
       MOVWF  TEMP1_L
       MOVLW  0
       MOVWF TEMP1_H
       GOTO  DELAY_MS
;=============================================
;ROTINA DE DELAY DE 4 MS
DELAY_4MS:
       MOVLW  0x04       ;DELAY DE 4 MS
       MOVWF  TEMP1_L
       MOVLW  0
       MOVWF TEMP1_H
       GOTO  DELAY_MS

;=============================================
DELAY_1MS:  ;(979,4 uS)
       MOVLW  1
       MOVWF  TEMP1_L
       MOVLW  0
       MOVWF TEMP1_H
       
;=============================================
;ROTINA DE DELAY EM MILISSEGUNDOS
; TEMP1_L/TEMP1_H TEM O TEMPO DESEJADO EM MS
;============================================
DELAY_MS:
       CLRF  CONT0
       CLRF   CONT1
DLY_001: 
       MOVLW  0x31
       MOVWF  TEMP1_L0
DLY_002: 
       MOVLW  0x1F
       MOVWF  TEMP1_L1
DLY_003: 
       DECFSZ TEMP1_L1,F
       GOTO   DLY_003
       DECFSZ TEMP1_L0,F
       GOTO   DLY_002
       MOVLW  0x28
       MOVWF  TEMP1_L0
DLY_004: 
       DECFSZ TEMP1_L0,F
       GOTO   DLY_004
       INCF  CONT0,F
       BTFSC  STATUS,Z
       INCF   CONT1,F
       MOVF   TEMP1_L,W
       SUBWF CONT0,W
       BTFSS  STATUS,Z
       GOTO   DLY_001
       MOVF  TEMP1_H,W
       SUBWF  CONT1,W
       BTFSS  STATUS,Z
       GOTO   DLY_001
       RETURN


;***************************************************************
;                  INICIO DO PROGRAMA
;***************************************************************

START:
       CALL   PORTS_INIT  ;INICIALIZA PORTS
       CALL   LCD_INIT    ;INICIALIZA LCD
       #IFDEF CAP_PINO6   ;OPO DO USURIO
       BTFSC  V_BAT       ;TESTA PINO 6
       GOTO   INICIO      ;FOI BOTO RESET APERTADO
       #ENDIF             ;FOI INTERRUPTOR ON/OFF
       CALL   MENSAGEM_NAME; MENSAGEM 'PIC_CAPACIMETER'
       CALL   MENSAGEM_BLOG ;MENSAGEM 'BLOG PICSOURCE'
       CALL   DELAY_1SEG    ;MOSTRA MENSAGEM POR 1 SEGUNDO 
INICIO: 
       BCF    FLAG_RS  ;DEFINE ENVIO DE COMANDO (RS=0)
       MOVLW  1        ;COMANDO PARA LIMPAR TELA DO LCD
       MOVWF  DADO
       CALL   ENV_BYTE ;ENVIA COMANDO 'LIMPAR DISPLAY'
       BSF    RESET_555 ;PINO 4 DO 555 EM '1'
       BSF    DISP_555  ;PINO 2 DO 555 EM '1'
       CLRF   RES1_L ;ZERA REGISTRADORES
       CLRF   RES1_H
       CLRF   N_LEITURAS
       CALL   MEDIR_CAP_REF ;MEDIO INICIAL PARA AUTOZERO
MAIN_1: 
       CLRF   ESCALA   ;INICIA ASSINALANDO MEDIO EM PF
       CALL   MEDIR_CAPACITOR    ;INICIA A MEDIO DO CAPACITOR
       MOVF  ESCALA,W  ;RECUPERA SITUAO DA ESCALA
       BTFSS  STATUS,Z ;TESTA SE FICOU EM ESCALA PF
       GOTO   MOSTRA_RESULTADO ;MUDOU DE ESCALA NA LEITURA
       MOVF   RES_L,W      ;SALVA A PRIMEIRA MEDIO SE ESTAVA EM PF
       MOVWF  RES_L_SALVO
       MOVF  RES_H,W
       MOVWF  RES_H_SALVO
       CALL   DELAY_10MS ;DELAY DE 10 MS
       CALL   MEDIR_CAPACITOR ;INICIA NOVA MEDIO DO CAPACITOR
       MOVF   RES_L,W     ;SALVA A SEGUNDA MEDIO
       MOVWF  RES2_L_SALVO
       MOVF   RES_H,W
       MOVWF  RES2_H_SALVO
       CALL   DELAY_10MS ;DELAY DE 10 MS      

;=======================================================================
;SOMA A PRIMEIRA MEDIO COM A SEGUNDA E DIVIDE POR 2 (MDIA)
       MOVF   RES_L_SALVO,W
       MOVWF  TEMP1_L
       MOVF   RES_H_SALVO,W
       MOVWF TEMP1_H
       MOVF   RES2_L_SALVO,W
       ADDWF  TEMP1_L,F
       BTFSC  STATUS,C
       INCF  TEMP1_H,F
       MOVF   RES2_H_SALVO,W
       ADDWF TEMP1_H,F
       MOVLW  2           ;DIVIDE POR 2 (0X02)
       MOVWF CONT0
       MOVLW  0
       MOVWF  CONT1
       CALL   FAZ_DIVISAO      ;DIVISO
       MOVF   TEMP1_L,W
       MOVWF  RES_L            ;RESULTADO L DA MEDIO
       MOVF  TEMP1_H,W
       MOVWF  RES_H            ;RESULTADO H DA MEDIO
;========================================================================
;MOSTRA RESULTADO DA MEDIO NO LCD
MOSTRA_RESULTADO: 
       CALL   SHOW_RESULTADO   ;PROCESSA RESULTADO E ENVIA PARA LCD
       GOTO   MAIN_1           ;RETORNA AO LOOP MAIN

;========================================================================
;       FIM DO PROGRAMA
;========================================================================
       END
;========================================================================
